from argparse import ArgumentParser
from pathlib import Path

from daktari import __version__


def validate_as_file_path(parser: ArgumentParser, arg: str) -> Path:
    path = Path(arg)
    if not path.is_file():
        parser.error(f"Could not find config file {arg}")
    else:
        return path


argument_parser = ArgumentParser(description="Check developer environment configuration.")
argument_parser.add_argument("-d", "--debug", action="store_true", help="turn on debug logging")
argument_parser.add_argument(
    "-g", "--generate-local-config", action="store_true", help="generate a template file for local configuration"
)
argument_parser.add_argument(
    "-i", "--show-ignored", action="store_true", help="show checks affected by ignoredChecks local setting"
)
argument_parser.add_argument(
    "-c",
    "--config",
    default=".daktari.py",
    dest="config_path",
    required=False,
    help="Python configuration file",
    metavar="FILE",
    type=lambda arg: validate_as_file_path(argument_parser, arg),
)
argument_parser.add_argument("--version", action="version", version="%(prog)s {version}".format(version=__version__))



import logging
import os
import sys

from pyfiglet import Figlet

from daktari.check_runner import run_checks
from daktari.config import read_config, Config, write_local_config_template
from daktari.options import argument_parser


def print_logo(title: str):
    figlet = Figlet(font="slant")
    print(figlet.renderText(title))


def main() -> int:
    args = argument_parser.parse_args()
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    if args.generate_local_config:
        write_local_config_template()
        return 0

    config = read_config(args.config_path)
    if config is None:
        return 1

    os.chdir(args.config_path.parent.absolute())
    print_config_messages(config, args)

    all_passed = run_checks(config.checks)
    print("")
    return 0 if all_passed else 1


def print_config_messages(config: Config, args):
    if config.title:
        print_logo(config.title)

    ignored_count = len(config.ignored_checks)
    if ignored_count > 0:
        if args.show_ignored:
            print("ⓘ  The following checks have been ignored:\n")
            for check in config.ignored_checks:
                print(f"[{check.name}]")
            print("")
        else:
            print(f"ⓘ  {ignored_count} check(s) have been marked as ignored. Run with --show-ignored to list them.\n")


if __name__ == "__main__":
    sys.exit(main())



import unittest

from daktari.collection_utils import flatten


class TestCollectionUtils(unittest.TestCase):
    def test_flatten(self):
        original = [{"A", "B", "C"}, {"B", "D"}, {"A", "Z"}, {}]
        result = flatten(original)
        self.assertEqual({"A", "B", "C", "D", "Z"}, result)


if __name__ == "__main__":
    unittest.main()



import json
import logging
import os.path
from json.decoder import JSONDecodeError
from pathlib import Path
from typing import Optional

import dpath.util
from semver import VersionInfo

from daktari.check import Check, CheckResult
from daktari.checks.files import FilesExist
from daktari.checks.xml import XmlFileXPathCheck
from daktari.command_utils import CommandErrorException, run_command
from daktari.os import OS, detect_os
from daktari.version_utils import try_parse_semver, sanitise_version_string

BUNDLE_ID_INTELLIJ_IDEA = "com.jetbrains.intellij"
BUNDLE_ID_INTELLIJ_IDEA_CE = "com.jetbrains.intellij.ce"

SNAP_NAME_INTELLIJ_IDEA = "intellij-idea-ultimate"
SNAP_NAME_INTELLIJ_IDEA_CE = "intellij-idea-community"


def locate_intellij_idea_mac():
    from AppKit import NSWorkspace

    for bundle_id in (BUNDLE_ID_INTELLIJ_IDEA, BUNDLE_ID_INTELLIJ_IDEA_CE):
        url = NSWorkspace.sharedWorkspace().URLForApplicationWithBundleIdentifier_(bundle_id)
        if url is not None:
            logging.debug(f"IntelliJ IDEA location (via NSWorkspace): {url}")
            return url

    logging.debug("Could not find IntelliJ IDEA (via NSWorkspace)")
    return None


def get_intellij_idea_version_mac() -> Optional[VersionInfo]:
    intellij_url = locate_intellij_idea_mac()
    if intellij_url is None:
        return None
    else:
        from Foundation import NSBundle

        version_str = NSBundle.bundleWithURL_(intellij_url).objectForInfoDictionaryKey_("CFBundleShortVersionString")

        version_str = sanitise_version_string(version_str)
        version = try_parse_semver(version_str)

        logging.debug(f"IntelliJ IDEA version (via NSBundle): {version}")
        return version


def get_intellij_idea_version_snap() -> Optional[VersionInfo]:
    if not Path("/run/snapd.socket").is_socket():
        logging.debug("/run/snapd.socket does not exist, not querying snapd")
        return None

    from requests_unixsocket import Session

    session = Session()
    snaps_req = session.get(
        f"http+unix://%2Frun%2Fsnapd.socket/v2/snaps?snaps={SNAP_NAME_INTELLIJ_IDEA},{SNAP_NAME_INTELLIJ_IDEA_CE}"
    )
    snaps_info = snaps_req.json()
    logging.debug(f"response from snapd: {snaps_info}")
    version_str = dpath.util.get(snaps_info, "/result/0/version", default=None)
    logging.debug(f"raw snapd version: {version_str}")

    version_str = sanitise_version_string(version_str)

    version = try_parse_semver(version_str)
    logging.debug(f"IntelliJ IDEA version (via snapd): {version}")
    return version


def get_intellij_idea_version_tarball() -> Optional[VersionInfo]:
    try:
        idea_bin_path = run_command(["sh", "-c", "which idea.sh"]).stdout.rstrip("\n")
    except CommandErrorException:
        logging.debug("Could not locate idea.sh", exc_info=True)
        return None

    product_info_path = os.path.join(os.path.dirname(idea_bin_path), "..", "product-info.json")
    try:
        with open(product_info_path, "rb") as product_info_file:
            product_info = json.load(product_info_file)
    except IOError:
        logging.debug("Failed to read IntelliJ IDEA product-info.json", exc_info=True)
        return None
    except JSONDecodeError:
        logging.debug("Failed to parse IntelliJ IDEA product-info.json", exc_info=True)
        return None

    version_str = product_info.get("version", None)
    version = try_parse_semver(version_str)
    logging.debug(f"IntelliJ IDEA version (via product-info.json): {version}")
    return version


def get_intellij_idea_version() -> Optional[VersionInfo]:
    os = detect_os()
    if os == OS.OS_X:
        return get_intellij_idea_version_mac()
    elif os == OS.UBUNTU:
        return get_intellij_idea_version_snap() or get_intellij_idea_version_tarball()
    else:
        return get_intellij_idea_version_tarball()


class IntelliJIdeaInstalled(Check):
    name = "intellij.installed"

    suggestions = {OS.GENERIC: "Install IntelliJ Ultimate: https://www.jetbrains.com/idea/download/"}

    def __init__(self, required_version: Optional[str] = None, recommended_version: Optional[str] = None):
        self.required_version = required_version
        self.recommended_version = recommended_version

    def check(self) -> CheckResult:
        intellij_version = get_intellij_idea_version()
        return self.validate_semver_expression(
            "IntelliJ IDEA", intellij_version, self.required_version, self.recommended_version
        )


class IntelliJProjectImported(FilesExist):
    name = "intellij.projectImported"
    file_paths = [".idea/workspace.xml"]
    pass_fail_message = "Project <not/> imported into IntelliJ"
    depends_on = [IntelliJIdeaInstalled]
    suggestions = {
        OS.GENERIC: """
            From the IntelliJ start screen, click 'Open or Import' and choose the repository root directory
            """
    }


class IntelliJNodePackageManagerConfigured(XmlFileXPathCheck):
    name = "intellij.nodePackageManagerConfigured"
    file_path = ".idea/workspace.xml"
    xpath_query = "./component[@name='PropertiesComponent']"
    depends_on = [IntelliJProjectImported]

    def __init__(self, package_manager_path: str):
        self.package_manager_path = package_manager_path
        self.pass_fail_message = f"IntelliJ package manager has <not/> been set to {package_manager_path}"

        self.suggestions = {
            OS.GENERIC: f"""
                Follow the steps to configure {self.package_manager_path} as your package manager:
                https://www.jetbrains.com/help/idea/installing-and-removing-external-software-using-node-package-manager.html#ws_npm_yarn_set_yarn_default
                """
        }

    def validate_query_result(self, result):
        key_json = None if result is None else json.loads(result.text)
        logging.debug(f"Raw properties json: {key_json}")
        current_package_manager = str(key_json["keyToString"]["nodejs_package_manager_path"])
        logging.debug(f"IntelliJ node package manager set to: {current_package_manager}")
        return current_package_manager.__contains__(self.package_manager_path)



import json
import logging
import os.path
from json import JSONDecodeError

from daktari.check import Check, CheckResult
from daktari.command_utils import can_run_command
from daktari.file_utils import file_exists
from daktari.os import OS, detect_os


class GoogleCloudSdkInstalled(Check):
    name = "google.cloudSdkInstalled"

    suggestions = {
        OS.OS_X: """<cmd>brew install --cask google-cloud-sdk</cmd>

Then, add the gcloud components to your PATH.

For bash users, add this to ~/.bashrc:
source "$(brew --prefix)/Caskroom/google-cloud-sdk/latest/google-cloud-sdk/path.bash.inc"

For zsh users, add this to ~/.zhsrc:
source "$(brew --prefix)/Caskroom/google-cloud-sdk/latest/google-cloud-sdk/path.zsh.inc\" """,
        OS.UBUNTU: "<cmd>sudo snap install google-cloud-sdk --classic</cmd>",
        OS.GENERIC: "Install gcloud: https://cloud.google.com/sdk/docs/quickstart",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("gcloud --version"), "Google Cloud SDK is <not/> installed and on $PATH")


class CloudSqlProxyInstalled(Check):
    name = "google.cloudSqlProxyInstalled"
    depends_on = [GoogleCloudSdkInstalled]

    suggestions = {
        OS.GENERIC: "Install Cloud SQL Proxy: <cmd>gcloud components install cloud_sql_proxy</cmd>",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("cloud_sql_proxy --version"), "Cloud SQL Proxy is <not/> installed")


class GkeGcloudAuthPluginInstalled(Check):
    name = "google.gkeGcloudAuthPluginInstalled"
    depends_on = [GoogleCloudSdkInstalled]

    suggestions = {
        OS.UBUNTU: "<cmd>sudo apt-get install google-cloud-sdk-gke-gcloud-auth-plugin</cmd>",
        OS.GENERIC: "<cmd>gcloud components install gke-gcloud-auth-plugin</cmd>",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("gke-gcloud-auth-plugin --version "), "GKE auth plugin is <not/> installed")


class DockerGoogleCloudAuthConfigured(Check):
    name = "google.dockerGCloudAuthConfigured"
    depends_on = [GoogleCloudSdkInstalled]

    def __init__(self, cloud_project, region, region_number):
        self.suggestions = {
            OS.OS_X: f"""
                Setup gcloud auth and add gcloud auth helper to docker config. Run:
                <cmd>gcloud init</cmd>
                Select '{cloud_project}' as the cloud project.
                Select '{region}' as the region ({region_number}). Then run:
                <cmd>gcloud auth configure-docker</cmd>
                """,
            OS.GENERIC: f"""
                On Linux you should be running docker with sudo.
                If you haven't already initialise gcloud as root:
                <cmd>sudo gcloud init</cmd>
                Select '{cloud_project}' as the cloud project.
                Select '{region}' as the region ({region_number}).
                Then create the docker config as root:
                <cmd>sudo gcloud auth configure-docker</cmd>
                """,
        }

    def check(self) -> CheckResult:
        if detect_os() != OS.OS_X:
            return self.passed_with_warning("Not checking docker gcloud configuration as it might require sudo")

        docker_config_path = os.path.expanduser("~/.docker/config.json")
        if not file_exists(docker_config_path):
            return self.failed(f"{docker_config_path} does not exist")

        try:
            with open(docker_config_path, "rb") as docker_config_file:
                docker_config = json.load(docker_config_file)
        except IOError:
            logging.error(f"Exception reading {docker_config_path}", exc_info=True)
            return self.failed(f"Failed to read {docker_config_path}")
        except JSONDecodeError:
            logging.error(f"Exception parsing {docker_config_path}", exc_info=True)
            return self.failed(f"Failed to parse {docker_config_path}")

        if docker_config.get("credHelpers", {}).get("gcr.io") != "gcloud":
            return self.failed("docker gcloud auth not configured")

        return self.passed("docker gcloud auth configured")



from daktari.check import Check, CheckResult


class DummyCheck(Check):
    def __init__(self, name: str = "check.name", depends_on=None):
        if depends_on is None:
            depends_on = []
        self.name = name
        self.depends_on = depends_on

    def check(self) -> CheckResult:
        return self.passed("no check")



from typing import Dict, List, Set, TypeVar

from daktari.check import Check
from daktari.check_utils import get_all_dependent_check_names


def dependency_graph(checks: List[Check]) -> Dict[str, Set[str]]:
    return {check.name: get_all_dependent_check_names(check) for check in checks}


T = TypeVar("T")


def stable_topological_sort(items: List[T], dependencies: Dict[T, Set[T]]) -> List[T]:
    result = items.copy()
    for i in range(len(items)):
        for j in range(i):
            if result[i] in dependencies[result[j]]:
                item = result[i]
                del result[i]
                result.insert(j, item)
    return result


def sort_checks(checks: List[Check]) -> List[Check]:
    graph = dependency_graph(checks)
    check_names = [check.name for check in checks]
    sorted_check_names = stable_topological_sort(check_names, graph)

    def get_topo_order(check: Check):
        return sorted_check_names.index(check.name)

    return sorted(checks, key=get_topo_order)



import logging
from typing import Optional

from semver import VersionInfo

from daktari.command_utils import get_stdout


def get_simple_cli_version(binary_name: str) -> Optional[VersionInfo]:
    raw_version = get_stdout(f"{binary_name} --version")
    if raw_version:
        version = try_parse_semver(raw_version)
        logging.debug(f"{binary_name} version: {version}")
        return version
    return None


def try_parse_semver(version_str: Optional[str]) -> Optional[VersionInfo]:
    try:
        return None if version_str is None else VersionInfo.parse(version_str)
    except ValueError:
        return None


def sanitise_version_string(version_str: str) -> str:
    return version_str + ".0" if version_str.count(".") == 1 else version_str



import logging

from daktari.check import Check, CheckResult
from daktari.command_utils import can_run_command, get_stdout
from daktari.file_utils import file_contains_text, is_ascii
from daktari.os import OS


class GitInstalled(Check):
    name = "git.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install git</cmd>",
        OS.UBUNTU: "<cmd>sudo apt install git</cmd>",
        OS.GENERIC: "Install Git: https://git-scm.com/downloads",
    }

    def check(self) -> CheckResult:
        if can_run_command("git version"):
            return self.passed("Git is installed")
        else:
            return self.failed("Could not find git on the path")


class GitLfsInstalled(Check):
    name = "git.lfs.installed"
    depends_on = [GitInstalled]

    suggestions = {
        OS.OS_X: "<cmd>brew install git-lfs</cmd>",
        OS.UBUNTU: "<cmd>sudo apt install git-lfs</cmd>",
        OS.GENERIC: "Install Git LFS: https://github.com/git-lfs/git-lfs/wiki/Installation",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("git lfs version"), "Git LFS is <not/> installed")


class GitLfsSetUpForUser(Check):
    name = "git.lfs.setUpForUser"
    depends_on = [GitLfsInstalled]

    suggestions = {
        OS.GENERIC: """
            Set up Git LFS for your user account:
            <cmd>git lfs install</cmd>
            """
    }

    def check(self) -> CheckResult:
        output = get_stdout("git lfs env")
        passed = bool(output and "git-lfs filter-process" in output)
        return self.verify(passed, "Git LFS is <not/> set up for the current user")


class GitLfsFilesDownloaded(Check):
    name = "git.lfs.filesDownloaded"
    depends_on = [GitLfsSetUpForUser]

    suggestions = {
        OS.GENERIC: """
            Download all Git LFS files and update working copy with the downloaded content:
            <cmd>git lfs pull</cmd>
            """
    }

    def check(self) -> CheckResult:
        output = get_stdout("git lfs ls-files") or ""
        files_not_downloaded = [line.split()[2] for line in output.splitlines() if line.split()[1] == "-"]
        for file in files_not_downloaded:
            logging.info(f"Git LFS file not downloaded: {file}")
        passed = len(files_not_downloaded) == 0
        return self.verify(passed, "Git LFS files have <not/> been downloaded")


class GitCryptInstalled(Check):
    name = "git.crypt.installed"
    depends_on = [GitInstalled]

    suggestions = {
        OS.OS_X: "<cmd>brew install git-crypt</cmd>",
        OS.UBUNTU: "<cmd>sudo apt install git-crypt</cmd>",
        OS.GENERIC: "Install git-crypt: https://www.agwa.name/projects/git-crypt/",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("git crypt version"), "git-crypt is <not/> installed")


class GitCryptUnlocked(Check):
    name = "git.crypt.unlocked"
    depends_on = [GitCryptInstalled]

    def __init__(self, fileToCheck: str):
        self.fileToCheck = fileToCheck

    suggestions = {
        OS.GENERIC: """
            Unlock this repository with:
            <cmd>git-crypt unlock</cmd>
            """,
    }

    def check(self) -> CheckResult:
        is_unlocked = is_ascii(self.fileToCheck)
        return self.verify(is_unlocked, "Encrypted files have <not/> been unlocked")


class PreCommitInstalled(Check):
    name = "preCommit.installed"
    depends_on = [GitInstalled]

    suggestions = {
        OS.OS_X: "<cmd>brew install pre-commit</cmd>",
        OS.GENERIC: "Install pre-commit: https://pre-commit.com/#installation",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("pre-commit --version"), "pre-commit is <not/> installed")


class PreCommitGitHooksInstalled(Check):
    name = "preCommit.gitHooksInstalled"
    depends_on = [PreCommitInstalled]

    suggestions = {
        OS.GENERIC: "<cmd>pre-commit install</cmd>",
    }

    def check(self) -> CheckResult:
        git_hooks_installed = file_contains_text(".git/hooks/pre-commit", "pre-commit.com")
        return self.verify(git_hooks_installed, "pre-commit Git hooks are <not/> installed")


class GpgInstalled(Check):
    name = "gpg.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install gpg2 gnupg pinentry-mac</cmd>",
        OS.UBUNTU: "<cmd>sudo apt install gpg</cmd>",
        OS.GENERIC: "Install gpg: https://gnupg.org/",
    }

    def check(self) -> CheckResult:
        return self.verify_install("gpg")


class GitCommitSigningSetUp(Check):
    name = "git.commitSigningSetUp"
    depends_on = [GpgInstalled]

    suggestions = {
        OS.OS_X: "Follow instructions to set up commit signing: "
        "https://gist.github.com/troyfontaine/18c9146295168ee9ca2b30c00bd1b41e#file-2-using-gpg-md",
        OS.UBUNTU: "Follow instructions to set up commit signing: "
        "https://brain2life.hashnode.dev/how-to-sign-your-git-commits-in-ubuntu-2004-and-why-you-need-it",
    }

    def check(self) -> CheckResult:
        key = get_stdout("git config user.signingkey")
        passed = key is not None and key != ""
        return self.verify(passed, "user.signingkey is <not/> set")


class GitCommitAutoSigningEnabled(Check):
    name = "git.commitAutoSigningEnabled"
    depends_on = [GitCommitSigningSetUp]

    suggestions = {OS.GENERIC: "<cmd>git config commit.gpgsign true</cmd>"}

    def check(self) -> CheckResult:
        setting = get_stdout("git config commit.gpgsign")
        passed = setting is not None and setting.rstrip() == "true"
        return self.verify(passed, "commit.gpgsign is <not/> enabled")
